@@ -2,6 +2,7 @@ import inspect
 import itertools
 import sys
 import six
+from uuid import uuid4
 
 from time import time
 from types import FunctionType, MethodType
@@ -33,6 +34,7 @@ class TimedObject(object):
 class CaseWrapper(TimedObject):
     def __init__(self, case_func, parent, execute_kwargs=None, metadata={}):
         super(CaseWrapper, self).__init__()
+        self.id = str(uuid4())
         self.case_func = case_func
         self.expects = []
         self.parent = parent
@@ -52,11 +54,19 @@ class CaseWrapper(TimedObject):
         self.start()
         try:
             MethodType(self.case_func, context or self)(**kwargs)
-        except TestIncompleteException:
+        except TestIncompleteException as e:
             self.incomplete = True
+
+            # If thrown during decorators
+            if e.real_func:
+                self.case_func = e.real_func
         except TestSkippedException as e:
             self.skipped = True
             self.skip_reason = e.reason if type(e.reason) is str else ''
+
+            # If thrown during decorators
+            if e.real_func:
+                self.case_func = e.real_func
         except FailedRequireException:
             pass
         except Exception as e:
@@ -77,24 +87,66 @@ class CaseWrapper(TimedObject):
 
     @property
     def success(self):
-        return (not self.failed and not self.error and
+        return (self.complete and not self.failed and not self.error and
                 len([exp for exp in self.expects if not exp.success]) == 0)
 
+    @property
+    def complete(self):
+        return self.end_time > 0.0
+
+    def __getstate__(self):
+        altered = dict(self.__dict__)
+        if 'case_func' in altered:
+            altered['case_func'] = self.id
+        if 'parent' in altered:
+            altered['parent'] = self.parent.id
+        return altered
+
+    def __eq__(self, other):
+        if isinstance(other, CaseWrapper):
+            return self.id == other.id
+        return False
+
+    def __ne__(self, other):
+        return not self == other
+
 
 class Describe(EventDispatcher):
     __FIXTURE__ = False
 
     def __init__(self, parent=None):
         super(Describe, self).__init__()
+        self.id = str(uuid4())
+        wrappers = self.__wrappers__
         self.parent = parent
-        self.cases = self.__wrappers__
+        self.cases = wrappers
         self.describes = [desc_type(parent=self)
                           for desc_type in self.describe_types]
+        self._num_completed_cases = 0
+        self._state = self.__create_state_obj__()
 
     @property
     def name(self):
         return convert_camelcase(self.__class__.__name__)
 
+    @property
+    def complete(self):
+        cases_completed = self._num_completed_cases == len(self.cases)
+        descs_not_completed = [desc for desc in self.describes
+                               if not desc.complete]
+        return cases_completed and len(descs_not_completed) == 0
+
+    @property
+    def real_class_path(self):
+        ancestors = [self.__class__.__name__]
+        parent = self.parent
+        while parent:
+            ancestors.insert(0, parent.__class__.__name__)
+            parent = parent.parent
+
+        real_path = '.'.join(ancestors)
+        return '{base}.{path}'.format(base=self.__module__, path=real_path)
+
     @property
     def doc(self):
         return type(self).__doc__
@@ -102,7 +154,7 @@ class Describe(EventDispatcher):
     @property
     def total_time(self):
         total = 0.0
-        for case in self.cases:
+        for key, case in six.iteritems(self.cases):
             total += case.elapsed_time
 
         for describe in self.describes:
@@ -113,7 +165,8 @@ class Describe(EventDispatcher):
     @property
     def success(self):
         ok = True
-        case_successes = [case.success for case in self.cases]
+        case_successes = [case.success
+                          for key, case in six.iteritems(self.cases)]
         spec_successes = [spec.success for spec in self.describes]
         if case_successes and False in case_successes:
             ok = False
@@ -124,11 +177,11 @@ class Describe(EventDispatcher):
 
     @property
     def __wrappers__(self):
-        wrappers = []
+        wrappers = {}
         for case_func in self.case_funcs:
             case_func, metadata = extract_metadata(case_func)
-            wrappers.append(CaseWrapper(case_func, parent=self,
-                                        metadata=metadata))
+            wrapper = CaseWrapper(case_func, parent=self, metadata=metadata)
+            wrappers[wrapper.id] = wrapper
         return wrappers
 
     @classmethod
@@ -181,20 +234,74 @@ class Describe(EventDispatcher):
     def is_fixture(cls):
         return vars(cls).get('__FIXTURE__') is True
 
-    def before_all(self):
-        pass
+    def __create_state_obj__(self):
+        """ Generates the clean state object magic. Here be dragons! """
+        stops = [Describe, Spec, DataDescribe, EventDispatcher]
 
-    def after_all(self):
-        pass
+        mros = [mro for mro in inspect.getmro(type(self)) if mro not in stops]
+        mros.reverse()
 
-    def before_each(self):
-        pass
+        # Create generic object
+        class GenericStateObj(object):
+            def before_all(self):
+                pass
 
-    def after_each(self):
-        pass
+            def after_all(self):
+                pass
 
-    def execute(self, select_metadata=None):
+            def before_each(self):
+                pass
 
+            def after_each(self):
+                pass
+
+        # Duplicate inheritance chain
+        chain = [GenericStateObj]
+        for mro in mros:
+            cls_name = '{0}StateObj'.format(mro.__name__)
+            cls = type(cls_name, (chain[-1:][0],), dict(mro.__dict__))
+            cls.__spec__ = self
+
+            chain.append(cls)
+
+        # Removing fallback
+        chain.pop(0)
+
+        state_cls = chain[-1:][0]
+        return state_cls()
+
+    def parallel_execution(self, manager, select_metadata=None):
+        self.top_parent.dispatch(DescribeEvent(DescribeEvent.START, self))
+        self._state.before_all()
+
+        for key, case in six.iteritems(self.cases):
+            manager.add_to_queue(case)
+
+        for describe in self.describes:
+            describe.execute(select_metadata, manager)
+
+    def standard_execution(self, select_metadata=None):
+        self.top_parent.dispatch(DescribeEvent(DescribeEvent.START, self))
+        self._state.before_all()
+
+        # Execute Cases
+        for key, case in six.iteritems(self.cases):
+            self._state.before_each()
+            case.execute(context=self._state)
+            self._state.after_each()
+            self._num_completed_cases += 1
+
+            self.top_parent.dispatch(TestEvent(case))
+
+        # Execute Suites
+        for describe in self.describes:
+            describe.execute(select_metadata=select_metadata)
+
+        self._state.after_all()
+
+        self.top_parent.dispatch(DescribeEvent(DescribeEvent.COMPLETE, self))
+
+    def execute(self, select_metadata=None, parallel_manager=None):
         if select_metadata:
             self.cases = find_by_metadata(select_metadata, self.cases)
             self.describes = children_with_tests_with_metadata(
@@ -203,25 +310,10 @@ class Describe(EventDispatcher):
         # If it doesn't have tests or describes don't run it
         if len(self.cases) <= 0 and len(self.describes) <= 0:
             return
-
-        self.top_parent.dispatch(DescribeEvent(DescribeEvent.START, self))
-
-        self.before_all()
-
-        # Execute Cases
-        for case in self.cases:
-            self.before_each()
-            case.execute(context=self)
-            self.after_each()
-
-            self.top_parent.dispatch(TestEvent(case))
-
-        # Execute Suites
-        for describe in self.describes:
-            describe.execute(select_metadata=select_metadata)
-
-        self.after_all()
-        self.top_parent.dispatch(DescribeEvent(DescribeEvent.COMPLETE, self))
+        if parallel_manager:
+            self.parallel_execution(parallel_manager, select_metadata)
+        else:
+            self.standard_execution(select_metadata)
 
     @classmethod
     def plugin_filter(cls, other):
@@ -234,6 +326,7 @@ class Describe(EventDispatcher):
         return (issubclass(other, Describe) and
                 other is not cls
                 and other is not Spec
+                and other is not DataSpec
                 and other is not DataDescribe)
 
     @classmethod
@@ -241,13 +334,12 @@ class Describe(EventDispatcher):
         if type(obj) is not FunctionType:
             return False
 
+        reserved = ['execute', 'standard_execution', 'parallel_execution',
+                    'before_each', 'after_each', 'before_all', 'after_all']
+
         func_name = obj.__name__
         return (not func_name.startswith('_') and
-                not func_name == 'execute' and
-                not func_name == 'before_each' and
-                not func_name == 'after_each' and
-                not func_name == 'before_all' and
-                not func_name == 'after_all')
+                not func_name in reserved)
 
 
 class Spec(Describe):
@@ -259,7 +351,7 @@ class DataDescribe(Describe):
 
     def __init__(self, parent=None):
         super(DataDescribe, self).__init__(parent=parent)
-        self.cases = []
+        self.cases = {}
 
         self.dup_count = 0
         self.dups = set()
@@ -292,15 +384,20 @@ class DataDescribe(Describe):
 
                 # Monkey-patch and add to cases list
                 setattr(self, func_name, new_func)
-                self.cases.append(CaseWrapper(new_func, parent=self,
-                                              execute_kwargs=kwargs,
-                                              metadata=meta))
+                wrapper = CaseWrapper(new_func, parent=self,
+                                      execute_kwargs=kwargs, metadata=meta)
+                self.cases[wrapper.id] = wrapper
+
+
+class DataSpec(DataDescribe):
+    pass
 
 
 def convert_to_hashable(obj):
     hashed = obj
     if isinstance(obj, dict):
-        hashed = tuple([(k, convert_to_hashable(v)) for k, v in six.iteritems(obj)])
+        hashed = tuple([(k, convert_to_hashable(v))
+                       for k, v in six.iteritems(obj)])
     elif isinstance(obj, list):
         hashed = tuple(obj)
     return hashed
@@ -321,6 +418,7 @@ def copy_function(func, name):
 
     return FunctionType(code, globals, name)
 
+
 def get_function_kwargs(old_func, new_args):
     args, _, _, defaults = inspect.getargspec(old_func)
     if 'self' in args:
@@ -335,6 +433,7 @@ def get_function_kwargs(old_func, new_args):
     kwargs.update(new_args)
     return kwargs
 
+
 class DescribeEvent(Event):
     START = 'start'
     COMPLETE = 'complete'
@@ -352,12 +451,16 @@ class FailedRequireException(Exception):
 
 
 class TestSkippedException(Exception):
-    def __init__(self, func, reason=None):
+    def __init__(self, func, reason=None, other_data={}):
         self.func = func
         self.reason = reason
+        self.other_data = other_data
+        self.real_func = other_data.get('real_func')
 
 
 class TestIncompleteException(Exception):
-    def __init__(self, func, reason=None):
+    def __init__(self, func, reason=None, other_data={}):
         self.func = func
         self.reason = reason
+        self.other_data = other_data
+        self.real_func = other_data.get('real_func')
