@@ -4,6 +4,8 @@ import itertools
 import sys
 import six
 
+CAPTURED_TRACEBACKS = []
+
 
 def convert_camelcase(input_str):
     if input_str is None:
@@ -88,6 +90,11 @@ def get_real_last_traceback(exception):
     _n, _n, exc_traceback = sys.exc_info()
     tb_list = get_all_tracebacks(exc_traceback)[1:]
 
+    # Remove already captured tracebacks
+    # TODO(jmv): This must be a better way of doing this. Need to revisit.
+    tb_list = [tb for tb in tb_list if tb not in CAPTURED_TRACEBACKS]
+    CAPTURED_TRACEBACKS.extend(tb_list)
+
     for traceback in tb_list:
         lines, path, line_num = get_source_from_frame(traceback.tb_frame)
         traceback_lines = get_numbered_source(lines, traceback.tb_lineno,
@@ -100,7 +107,8 @@ def get_real_last_traceback(exception):
 
     traced_lines = ['Error Traceback:']
     traced_lines.extend(itertools.chain.from_iterable(traceback_blocks))
-    traced_lines.append('  - Error: {0}'.format(exception))
+    traced_lines.append('  - Error | {0}: {1}'.format(
+        type(exception).__name__, exception))
 
     return traced_lines
 
